#include "stdafx.h"
#include "NLayoutResourceLoader.h"

#include "NByteArray.h"
#include "XMLUtil.h"

#include "NPFileData.h"
#include "NPMemoryData.h"
#include <atlpath.h>

static const TCHAR	LAYOUT[] = _T("layout");
static const TCHAR  ITEM[] = _T("item");
static const TCHAR	ID[] = _T("id");
static const TCHAR	X[] = _T("x");
static const TCHAR	Y[] = _T("y");
static const TCHAR	WIDTH[] = _T("w");
static const TCHAR	HEIGHT[] = _T("h");

static const TCHAR	SCRIPTLET[] = _T("scriptlet");

CNLayoutResourceLoader::CNLayoutResourceLoader()
{

}

CNLayoutResourceLoader::~CNLayoutResourceLoader()
{
	
}
BOOL CNLayoutResourceLoader::Init(CNPUnzipFile* zipimage)
{
	__super::Init(zipimage);

	m_ZipImage->GoToFirstFile();

	UnzipFileInfo fileInfo;
	char szCurrentFileName[_MAX_PATH+1];

	BOOL bNextFile = TRUE;
	while (bNextFile)
	{
		m_ZipImage->GetCurrentFileInfo(&fileInfo, szCurrentFileName, _MAX_PATH, NULL, 0, NULL, 0);
		if (strncmp("layout", szCurrentFileName, strlen("binary")) == 0 &&
			strcmp(".xml", ATLPath::FindExtension(szCurrentFileName)) == 0)
		{
			LONG lFileSize = fileInfo.m_lUncompressedSize;
			if (lFileSize > 0)
			{
				CNByteArray bufArray;
				ReadCurrentZipFile(bufArray, lFileSize);

				if (bufArray.GetSize() > 0)
				{
					_ParseXML(bufArray.GetBuffer());
				}
			}
		}
		bNextFile = m_ZipImage->GoToNextFile();
	}
	m_ZipImage->GoToFirstFile();

	return TRUE;
}
BOOL CNLayoutResourceLoader::Close()
{	
	m_mapLayout.RemoveAll();
	// [:2008-12-16] Memory Leak 
	if (m_lstDocument.GetCount())
	{
		POSITION pos = m_lstDocument.GetHeadPosition();
		while (pos)
			delete m_lstDocument.GetNext(pos);		
	}
	return TRUE;
}
BOOL CNLayoutResourceLoader::GetLayout(LPCTSTR pszcID, LPRECT pLayout)
{
	CString strKey = pszcID;
	CAtlMap< CString, CRect >::CPair* pPair = m_mapLayout.Lookup(strKey);
	if (pPair != NULL)
	{
		CRect rcLayout = pPair->m_value;
		CopyRect(pLayout, &rcLayout);
		return TRUE;
	}	
	return FALSE;
}
BOOL CNLayoutResourceLoader::_ParseXML(const LPBYTE pBuf)
{
	USES_CONVERSION;

	BOOL bSuccess = FALSE;
	TiXmlDocument* pDocument = new TiXmlDocument();
	m_lstDocument.AddTail(pDocument);

	pDocument->Parse(reinterpret_cast<LPCSTR>(pBuf));
	
	TiXmlElement* pRoot = pDocument->FirstChildElement(T2A(LAYOUT));
	if (!pRoot) return FALSE;
	
	TiXmlElement* pRootScriptlet = pRoot->FirstChildElement(T2A(SCRIPTLET));
	if (pRootScriptlet)
	{
		TiXmlElement* pNode = pRootScriptlet->FirstChildElement();
		while (pNode)
		{
			_ParseScriptletNode(pNode);
			pNode = pNode->NextSiblingElement();
		}
	}

	static const int MAX_ID = 32;
	
	LONG lX = 0;
	LONG lY = 0;
	LONG lW = 0;
	LONG lH = 0;
	TiXmlElement* pItemNode = pRoot->FirstChildElement(T2A(ITEM));
	while (pItemNode)
	{
		_ParseItemNode(pItemNode);

		pItemNode = pItemNode->NextSiblingElement(T2A(ITEM));
	}
	bSuccess = TRUE;
	return bSuccess;
}
BOOL CNLayoutResourceLoader::_ParseItemNode(TiXmlElement* pItemParentNode)
{
	USES_CONVERSION;

	CString szID;
	CRect rcLayout;
	_GetLayout(pItemParentNode, szID, rcLayout);
	m_mapLayout.SetAt(szID, rcLayout);
	//NPTRACE(_T("_ParseItemNode ID(%s), Layout(%d, %d, %d, %d)\n"), szID, rcLayout.left, rcLayout.top, rcLayout.Width(), rcLayout.Height());

	TiXmlElement* pItemNode = pItemParentNode->FirstChildElement(T2A(ITEM));
	while (pItemNode)
	{
		//_GetLayout(pItemNode, szID, rcLayout);
		//m_mapLayout.SetAt(CString(szID), rcLayout);
		_ParseItemNode(pItemNode);

		pItemNode = pItemNode->NextSiblingElement(T2A(ITEM));
	}
	return TRUE;
}
BOOL CNLayoutResourceLoader::_GetLayout(TiXmlElement* pItemNode, CString& strID, CRect& rcLayout)
{
	USES_CONVERSION;

	ATLASSERT(pItemNode);

	static const int MAX_ID = 32;

	TCHAR	szID[MAX_ID+1] = {0};	
	LONG lX = 0;
	LONG lY = 0;
	LONG lW = 0;
	LONG lH = 0;
	
	XMLUtil::GetAttrValue(pItemNode, T2A(ID), szID, MAX_ID);
	XMLUtil::GetAttrValue(pItemNode, T2A(X), lX);
	XMLUtil::GetAttrValue(pItemNode, T2A(Y), lY);
	XMLUtil::GetAttrValue(pItemNode, T2A(WIDTH), lW);
	XMLUtil::GetAttrValue(pItemNode, T2A(HEIGHT), lH);

	strID = szID;
	rcLayout.SetRect(lX, lY, lX+lW, lY+lH);

	return TRUE;
}
BOOL CNLayoutResourceLoader::_ParseScriptletNode(TiXmlElement* pNode)
{
	ATLASSERT(pNode);
	
	const int MAX_VALUE = 128;
	TCHAR szID[MAX_VALUE] = {0};
	if (XMLUtil::GetAttrValue(pNode, "id", szID, MAX_VALUE))
	{
		m_mapScriptlet.SetAt(szID, pNode);
	}

	LPCSTR pszcValue = pNode->Value();
	if (stricmp("window", pszcValue) == 0)
	{
		m_mapPadScriptlet.SetAt(szID, pNode);
	}
	TiXmlElement* pChildNode = pNode->FirstChildElement();
	while (pChildNode)
	{
		_ParseScriptletNode(pChildNode);
		pChildNode = pChildNode->NextSiblingElement();
	}
	return TRUE;
}
TiXmlElement* CNLayoutResourceLoader::GetPanelNode(LPCTSTR pszcID)
{
	CString strKey = pszcID;
	CAtlMap< CString, TiXmlElement* >::CPair* pPair = m_mapScriptlet.Lookup(strKey);
	if (pPair != NULL)
	{
		TiXmlElement* pNode = pPair->m_value;
		return pNode;
	}
	return NULL;
}
BOOL CNLayoutResourceLoader::GetNodeList(CAtlMap< CString, TiXmlElement* >& lstNode)
{
	POSITION pos = m_mapPadScriptlet.GetStartPosition();
	while (pos)
	{
		CAtlMap< CString, TiXmlElement* >::CPair* pPair = m_mapPadScriptlet.GetNext(pos);
		lstNode.SetAt(pPair->m_key, pPair->m_value);
	}
	return TRUE;
}